//-----------------------------------------------------------------------------
// CParticleSystem
//
// This class is responsible for controlling the particle system.
//
// Copyright (c) 2000 Andreas Jnsson
//-----------------------------------------------------------------------------
#ifndef PARTICLESYSTEM_H
#define PARTICLESYSTEM_H

#define D3D_OVERLOADS
#include <d3dx.h>

#include "renderer.h"

struct SParticle
{
	SParticle   *pNext;
	D3DXVECTOR3  Pos;
	D3DXVECTOR3  Vel;
	float        fEnergy;
};

class CParticleSystem
{
public:
	CParticleSystem();
	~CParticleSystem();

	HRESULT Initialize(D3DXCOLOR c);
	void	Update(float fDeltaTime);
	HRESULT Render(CRenderer *pRenderer);

	void SetEmitterPos(D3DVECTOR &Pos);
	void MoveEmitterPos(D3DVECTOR &Pos);
	void ToggleEmitter();
	int  GetNbrOfLiveParticles();

protected:
	// These are the attributes that define the behaviour of the system
	int        m_nMaxParticles;
	int        m_nColorTableSize;
	float      m_fRadius;
	float      m_fBirthInterval;
	float      m_fEnergyDropOff;
	float      m_fNextBirth;
	float      m_fBirthVelocity;
	float      m_fGravity;
	float      m_fBounce;

	bool       m_bIsEmitting;
	int	       m_nLiveParticles;

	D3DXCOLOR  m_Color;

	D3DVECTOR  m_EmitterPos;
	D3DVECTOR  m_TargetEmitterPos;

	SParticle *m_pFreeParticles;
	SParticle *m_pLiveParticles;

	SParticle *m_pParticlePool;

	D3DCOLOR  *m_pColorTable;
};

#endif