//-----------------------------------------------------------------------------
// CRenderer
//
// This class is a helper class for the particle systems so that they can 
// share the vertex buffer in an efficient way, and at the same time making
// the implementation easier to understand.
//
// Copyright (c) 2000 Andreas Jnsson
//-----------------------------------------------------------------------------
#ifndef RENDERER_H
#define RENDERER_H

#include <d3dx.h>

#define FVF_VERTEX (D3DFVF_XYZ|D3DFVF_DIFFUSE|D3DFVF_TEX1)
struct SVertex
{
	D3DXVECTOR3 v;
	D3DCOLOR    c;
	D3DXVECTOR2 t;
};

class CRenderer
{
public:
	CRenderer();

	HRESULT Initialize(ID3DXContext *pD3DX, int nMaxTriangles);
	HRESULT Terminate();

	HRESULT Begin();
	void    Vertex(D3DXVECTOR3 &v, D3DCOLOR &c, D3DXVECTOR2 &t);
	HRESULT End();

	ID3DXContext		   *m_pD3DX;
	IDirect3D7			   *m_pD3D;
	IDirect3DDevice7       *m_pD3DDev;

protected:
	IDirect3DVertexBuffer7 *m_pVertexBuffer;
	int                     m_nMaxBatchSize;
	int                     m_nBufferSize;
	int                     m_nVertexOffset;

	WORD                   *m_pwIndices;
	int						m_nMaxIndices;

	SVertex                *m_pVertices;
	int                     m_nNumVertices;
};

#endif