//-----------------------------------------------------------------------------
// CD3DApp
//
// This is a framework class for quickly creating Direct3D applications. Derive
// a new application class from this and implement your code there.
//
// To compile with this class you must link with:
// user32.lib gdi32.lib ddraw.lib d3dx.lib winmm.lib
//
// Copyright (c) 2000 Andreas Jnsson
//-----------------------------------------------------------------------------
#ifndef D3DAPP_H
#define D3DAPP_H

#define WIN32_LEAN_AND_MEAN // Don't include stuff that are rarely used
#include <windows.h>

#define  D3D_OVERLOADS  // This lets us use special code for the D3D datatypes
#include <d3dx.h>

#include "timer.h"

class CD3DApp
{
public:
	CD3DApp();

	HRESULT Create(HINSTANCE, char *szCmdLine);
	int     Run();

	void    ReportError(HRESULT hr, char *szFile, int nLine);

	// Set these before calling Create to 
	// change the appearance of the window
    char *m_szWindowTitle;
	bool  m_bFullscreen;
	int   m_nWidth;
	int   m_nHeight;
	int   m_nColorBits;
	int   m_nAlphaBits;
	int   m_nDepthBits;
	int   m_nStencilBits;
	bool  m_bShowStats;
	bool  m_bNoVSync;

protected:
	virtual HRESULT Initialize(char *szCmdLine);	
	virtual HRESULT Terminate();
	virtual HRESULT UpdateScene();
	virtual HRESULT RenderScene();
	virtual HRESULT ShowStats();

    virtual LRESULT MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );

	// Use this timer when updating the world
	CTimer            m_SceneTimer;

    ID3DXContext     *m_pD3DX;
    IDirect3DDevice7 *m_pD3DDev;

	//-------------------------------------------------------------------------
	// You shouldn't need to know about the following part
	//-------------------------------------------------------------------------
	
	HRESULT InitializeWindow();
	HRESULT InitializeD3D();
	HRESULT TerminateD3D();

	HRESULT Destroy();
	HRESULT IdleAction();
	HRESULT HandleModeChanges();
	HRESULT Pause(bool bPause);

	HINSTANCE     m_hInst;
	HWND          m_hWnd;

	bool          m_bReady;
	bool          m_bActive;
	bool          m_bSceneIsRunning;

    IDirectDraw7 *m_pDD;

	// This timer is used for computing the framerate
	CTimer        m_AppTimer;

private:
	static LRESULT CALLBACK WndProc( HWND, UINT, WPARAM, LPARAM );
};

#endif